package com.qmth.ops.api.controller.admin;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.core.solar.enums.OrgType;
import com.qmth.ops.api.constants.OpsApiConstants;
import com.qmth.ops.api.vo.OrgPageVO;
import com.qmth.ops.api.vo.OrgTypeVO;
import com.qmth.ops.api.vo.OrgVO;
import com.qmth.ops.api.vo.SuccessVO;
import com.qmth.ops.biz.domain.Org;
import com.qmth.ops.biz.query.OrgQuery;
import com.qmth.ops.biz.service.FileService;
import com.qmth.ops.biz.service.OrgService;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.Arrays;

@RestController
@RequestMapping(OpsApiConstants.ADMIN_URI_PREFIX + "/org")
public class OrgController {

    @Resource
    private OrgService orgService;

    @Resource
    private FileService fileService;

    @RequestMapping("/types")
    @Aac(auth = BOOL.FALSE)
    public Object getTypes() {
        return Arrays.stream(OrgType.values()).map(OrgTypeVO::new).toArray();
    }

    @PostMapping("/query")
    public OrgPageVO query(OrgQuery query) {
        return new OrgPageVO(orgService.findByQuery(query));
    }

    @PostMapping("/insert")
    public OrgVO insert(Org org, @RequestParam(required = false) MultipartFile logoFile,
            @RequestParam(required = false) String logoMd5) throws Exception {
        org = orgService.insert(org);
        if (logoFile != null && logoMd5 != null) {
            fileService.uploadOrgLogo(org, logoFile, logoMd5);
            org = orgService.update(org);
        }
        return new OrgVO(org);
    }

    @PostMapping("/update")
    public OrgVO update(Org org, @RequestParam(required = false) MultipartFile logoFile,
            @RequestParam(required = false) String logoMd5) throws Exception {
        org = orgService.update(org);
        if (logoFile != null && logoMd5 != null) {
            fileService.uploadOrgLogo(org, logoFile, logoMd5);
            org = orgService.update(org);
        }
        return new OrgVO(org);
    }

    @PostMapping("/toggle")
    public Object toggle(@RequestParam Long id, @RequestParam Boolean enable) {
        return new SuccessVO(orgService.toggle(id, enable));
    }
}
